/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 */

#include <internal.h>
#include <plugin.h>
#include <version.h>
#include <savedstatuses.h>
#include <dbus-maybe.h>
#include <dbus-bindings.h>

static void message(PurplePlugin* plugin,const char* msg)
{
    purple_notify_message(plugin,PURPLE_NOTIFY_MSG_INFO,"Gnome-Shell Connector",msg,NULL,NULL,NULL);
}

#define GNOME_AVAILABLE 0
#define GNOME_INVISIBLE 1
#define GNOME_BUSY 2
#define GNOME_IDLE 3
#define GNOME_NSTATUS 4

static int Purple2Gnome(PurpleStatusPrimitive purple_status)
{
    switch (purple_status)
    {
    case PURPLE_STATUS_OFFLINE:
    case PURPLE_STATUS_INVISIBLE:
        return GNOME_INVISIBLE;
    case PURPLE_STATUS_UNAVAILABLE:
    case PURPLE_STATUS_AWAY:
    case PURPLE_STATUS_EXTENDED_AWAY:
        return GNOME_BUSY;
    case PURPLE_STATUS_AVAILABLE:
        return GNOME_AVAILABLE;
    default:
        return GNOME_INVISIBLE;
    }
}

static PurpleStatusPrimitive Gnome2Purple(int gnome_status)
{
    switch (gnome_status)
    {
    case GNOME_AVAILABLE:
        return PURPLE_STATUS_AVAILABLE;
    case GNOME_INVISIBLE:
        return PURPLE_STATUS_INVISIBLE;
    case GNOME_BUSY:
        return PURPLE_STATUS_AWAY;
    case GNOME_IDLE:
        return PURPLE_STATUS_INVISIBLE;
    }
    return PURPLE_STATUS_INVISIBLE;
}

#define TITLESIZE 512

struct SPluginParams
{
    PurplePlugin* plugin;
    DBusGConnection* connection;
    char status[GNOME_NSTATUS][TITLESIZE];
};
typedef struct SPluginParams PluginParams;


static void InitPluginParams(PluginParams* params,PurplePlugin* plugin,DBusGConnection* connection)
{
    size_t i;
    params->plugin = plugin;
    params->connection = connection;
    for (i = 0; i < GNOME_NSTATUS; ++i)
        params->status[i][0] = '\0';
}

static void SetInGnome(PluginParams* params,int status)
{
    GError* error = NULL;
    GValue v = {0,{{0}}};
    DBusGProxy* presence = dbus_g_proxy_new_for_name(
        params->connection,
        "org.gnome.SessionManager",
        "/org/gnome/SessionManager/Presence",
        "org.freedesktop.DBus.Properties");
    g_value_init(&v,G_TYPE_UINT);
    g_value_set_uint(&v,status);
    if (!dbus_g_proxy_call(
        presence,"Set",&error,
        G_TYPE_STRING,"org.gnome.SessionManager.Presence",
        G_TYPE_STRING,"status",
        G_TYPE_VALUE,&v,G_TYPE_INVALID,G_TYPE_INVALID))
    {
        message(params->plugin,error->message);
        g_error_free(error);
    }
}

static void status_signal_handler(DBusGProxy* presence,guint gnome_status,PluginParams* params)
{
    PurpleSavedStatus* current_status = purple_savedstatus_get_current();
    guint current_pidgin_status = Purple2Gnome(purple_savedstatus_get_type(current_status));
    if (current_pidgin_status < GNOME_NSTATUS && !purple_savedstatus_is_transient(current_status))
    {
        const char* title = purple_savedstatus_get_title(current_status);
        if (title)
            strcpy(params->status[current_pidgin_status],title);
    }
    if (gnome_status >= GNOME_NSTATUS)
        return;
    if (current_pidgin_status != gnome_status)
    {
        PurpleSavedStatus* new_status = NULL;
        if (params->status[gnome_status][0])
            new_status = purple_savedstatus_find(params->status[gnome_status]);
        if (new_status)
            purple_savedstatus_activate(new_status);
        else
        {
            PurpleSavedStatus* newStatus = purple_savedstatus_new(NULL,Gnome2Purple(gnome_status));
            purple_savedstatus_activate(newStatus);
        }
    }
}

static gboolean plugin_load(PurplePlugin *plugin)
{
    DBusGProxy* presence;
    PurpleSavedStatus* currentStatus;
    GError* error = NULL;
    PluginParams* params = malloc(sizeof(PluginParams));
    plugin->extra = params;
    DBusGConnection* connection = dbus_g_bus_get(DBUS_BUS_SESSION,&error);
    if (!connection)
    {
        message(plugin,error->message);
        g_error_free(error);
        return FALSE;
    }
    InitPluginParams(params,plugin,connection);
    presence = dbus_g_proxy_new_for_name(
        connection,
        "org.gnome.SessionManager",
        "/org/gnome/SessionManager/Presence",
        "org.gnome.SessionManager.Presence");
    currentStatus = purple_savedstatus_get_current();
    SetInGnome(params,Purple2Gnome(purple_savedstatus_get_type(currentStatus)));
    dbus_g_proxy_add_signal(presence,"StatusChanged",G_TYPE_UINT,G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(presence,"StatusChanged",G_CALLBACK(status_signal_handler),params,NULL);
    return TRUE;
}

static gboolean plugin_unload(PurplePlugin *plugin)
{
    free(plugin->extra);
    return TRUE;
}

static PurplePluginInfo info =
{
    PURPLE_PLUGIN_MAGIC,
    PURPLE_MAJOR_VERSION,
    PURPLE_MINOR_VERSION,
    PURPLE_PLUGIN_STANDARD,
    NULL,                                             /**< ui_requirement */
    0,                                                /**< flags          */
    NULL,                                             /**< dependencies   */
    PURPLE_PRIORITY_DEFAULT,                          /**< priority       */
    "gtk-2925578-gnomeshell",                         /**< id             */
    N_("Gnome Shell Connector"),                      /**< name           */
    DISPLAY_VERSION,                                  /**< version        */
    N_("Connects Pidgin to Gnome Shell Status"),      /**  summary        */
    N_("Gnome Shell Connector"),                      /**  description    */
    "Wojtek Surówka",                                 /**< author         */
    "http://gnomeshpidgin.sourceforge.net",           /**< homepage       */
    plugin_load,                                      /**< load           */
    plugin_unload,                                    /**< unload         */
    NULL,                                             /**< destroy        */
    NULL,                                             /**< ui_info        */
    NULL,                                             /**< extra_info     */
    NULL,                                             /**< prefs_info     */
    NULL,
    /* padding */
    NULL,
    NULL,
    NULL,
    NULL
};

static void init_plugin(PurplePlugin *plugin)
{
}

PURPLE_INIT_PLUGIN(dbus_example, init_plugin, info)

